@@ -1,37 +1,41 @@
+"""Creates reST files corresponding to Python modules for code documentation.
+
+Parses a directory tree looking for Python modules and packages and creates
+ReST files appropriately to create code documentation with Sphinx.  It also
+creates a modules index (named modules.<suffix>).
+
+This is derived from the "sphinx-autopackage" script, which is:
+Copyright 2008 Société des arts technologiques (SAT),
+https://sat.qc.ca/
 """
-    sphinx.ext.apidoc
-    ~~~~~~~~~~~~~~~~~
 
-    Parses a directory tree looking for Python modules and packages and creates
-    ReST files appropriately to create code documentation with Sphinx.  It also
-    creates a modules index (named modules.<suffix>).
-
-    This is derived from the "sphinx-autopackage" script, which is:
-    Copyright 2008 Société des arts technologiques (SAT),
-    https://sat.qc.ca/
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import argparse
+import fnmatch
 import glob
 import locale
 import os
+import re
 import sys
 from copy import copy
-from fnmatch import fnmatch
 from importlib.machinery import EXTENSION_SUFFIXES
 from os import path
-from typing import Any, Generator, List, Tuple
+from typing import TYPE_CHECKING, Any
 
 import sphinx.locale
 from sphinx import __display_version__, package_dir
 from sphinx.cmd.quickstart import EXTENSIONS
 from sphinx.locale import __
+from sphinx.util import logging
 from sphinx.util.osutil import FileAvoidWrite, ensuredir
 from sphinx.util.template import ReSTRenderer
 
+if TYPE_CHECKING:
+    from collections.abc import Generator, Sequence
+
+logger = logging.getLogger(__name__)
+
 # automodule options
 if 'SPHINX_APIDOC_OPTIONS' in os.environ:
     OPTIONS = os.environ['SPHINX_APIDOC_OPTIONS'].split(',')
@@ -51,19 +55,18 @@ template_dir = path.join(package_dir, 'templates', 'apidoc')
 def is_initpy(filename: str) -> bool:
     """Check *filename* is __init__ file or not."""
     basename = path.basename(filename)
-    for suffix in sorted(PY_SUFFIXES, key=len, reverse=True):
-        if basename == '__init__' + suffix:
-            return True
-    else:
-        return False
+    return any(
+        basename == '__init__' + suffix
+        for suffix in sorted(PY_SUFFIXES, key=len, reverse=True)
+    )
 
 
-def module_join(*modnames: str) -> str:
+def module_join(*modnames: str | None) -> str:
     """Join module names with dots."""
     return '.'.join(filter(None, modnames))
 
 
-def is_packagedir(dirname: str = None, files: List[str] = None) -> bool:
+def is_packagedir(dirname: str | None = None, files: list[str] | None = None) -> bool:
     """Check given *files* contains __init__ file."""
     if files is None and dirname is None:
         return False
@@ -77,23 +80,23 @@ def write_file(name: str, text: str, opts: Any) -> None:
     """Write the output file for module/package <name>."""
     quiet = getattr(opts, 'quiet', None)
 
-    fname = path.join(opts.destdir, '%s.%s' % (name, opts.suffix))
+    fname = path.join(opts.destdir, f'{name}.{opts.suffix}')
     if opts.dryrun:
         if not quiet:
-            print(__('Would create file %s.') % fname)
+            logger.info(__('Would create file %s.'), fname)
         return
     if not opts.force and path.isfile(fname):
         if not quiet:
-            print(__('File %s already exists, skipping.') % fname)
+            logger.info(__('File %s already exists, skipping.'), fname)
     else:
         if not quiet:
-            print(__('Creating file %s.') % fname)
+            logger.info(__('Creating file %s.'), fname)
         with FileAvoidWrite(fname) as f:
             f.write(text)
 
 
-def create_module_file(package: str, basename: str, opts: Any,
-                       user_template_dir: str = None) -> None:
+def create_module_file(package: str | None, basename: str, opts: Any,
+                       user_template_dir: str | None = None) -> None:
     """Build the text of the file and write the file."""
     options = copy(OPTIONS)
     if opts.includeprivate and 'private-members' not in options:
@@ -106,13 +109,20 @@ def create_module_file(package: str, basename: str, opts: Any,
         'qualname': qualname,
         'automodule_options': options,
     }
-    text = ReSTRenderer([user_template_dir, template_dir]).render('module.rst_t', context)
+    if user_template_dir is not None:
+        template_path = [user_template_dir, template_dir]
+    else:
+        template_path = [template_dir]
+    text = ReSTRenderer(template_path).render('module.rst_t', context)
     write_file(qualname, text, opts)
 
 
-def create_package_file(root: str, master_package: str, subroot: str, py_files: List[str],
-                        opts: Any, subs: List[str], is_namespace: bool,
-                        excludes: List[str] = [], user_template_dir: str = None) -> None:
+def create_package_file(root: str, master_package: str | None, subroot: str,
+                        py_files: list[str],
+                        opts: Any, subs: list[str], is_namespace: bool,
+                        excludes: Sequence[re.Pattern[str]] = (),
+                        user_template_dir: str | None = None,
+                        ) -> None:
     """Build the text of the file and write the file."""
     # build a list of sub packages (directories containing an __init__ file)
     subpackages = [module_join(master_package, subroot, pkgname)
@@ -122,6 +132,7 @@ def create_package_file(root: str, master_package: str, subroot: str, py_files:
     submodules = [sub.split('.')[0] for sub in py_files
                   if not is_skipped_module(path.join(root, sub), opts, excludes) and
                   not is_initpy(sub)]
+    submodules = sorted(set(submodules))
     submodules = [module_join(master_package, subroot, modname)
                   for modname in submodules]
     options = copy(OPTIONS)
@@ -140,7 +151,11 @@ def create_package_file(root: str, master_package: str, subroot: str, py_files:
         'show_headings': not opts.noheadings,
         'maxdepth': opts.maxdepth,
     }
-    text = ReSTRenderer([user_template_dir, template_dir]).render('package.rst_t', context)
+    if user_template_dir is not None:
+        template_path = [user_template_dir, template_dir]
+    else:
+        template_path = [template_dir]
+    text = ReSTRenderer(template_path).render('package.rst_t', context)
     write_file(pkgname, text, opts)
 
     if submodules and opts.separatemodules:
@@ -148,8 +163,8 @@ def create_package_file(root: str, master_package: str, subroot: str, py_files:
             create_module_file(None, submodule, opts, user_template_dir)
 
 
-def create_modules_toc_file(modules: List[str], opts: Any, name: str = 'modules',
-                            user_template_dir: str = None) -> None:
+def create_modules_toc_file(modules: list[str], opts: Any, name: str = 'modules',
+                            user_template_dir: str | None = None) -> None:
     """Create the module's index."""
     modules.sort()
     prev_module = ''
@@ -165,11 +180,16 @@ def create_modules_toc_file(modules: List[str], opts: Any, name: str = 'modules'
         'maxdepth': opts.maxdepth,
         'docnames': modules,
     }
-    text = ReSTRenderer([user_template_dir, template_dir]).render('toc.rst_t', context)
+    if user_template_dir is not None:
+        template_path = [user_template_dir, template_dir]
+    else:
+        template_path = [template_dir]
+    text = ReSTRenderer(template_path).render('toc.rst_t', context)
     write_file(name, text, opts)
 
 
-def is_skipped_package(dirname: str, opts: Any, excludes: List[str] = []) -> bool:
+def is_skipped_package(dirname: str, opts: Any,
+                       excludes: Sequence[re.Pattern[str]] = ()) -> bool:
     """Check if we want to skip this module."""
     if not path.isdir(dirname):
         return False
@@ -177,31 +197,26 @@ def is_skipped_package(dirname: str, opts: Any, excludes: List[str] = []) -> boo
     files = glob.glob(path.join(dirname, '*.py'))
     regular_package = any(f for f in files if is_initpy(f))
     if not regular_package and not opts.implicit_namespaces:
-        # *dirname* is not both a regular package and an implicit namespace pacage
+        # *dirname* is not both a regular package and an implicit namespace package
         return True
 
     # Check there is some showable module inside package
-    if all(is_excluded(path.join(dirname, f), excludes) for f in files):
-        # all submodules are excluded
-        return True
-    else:
-        return False
+    return all(is_excluded(path.join(dirname, f), excludes) for f in files)
 
 
-def is_skipped_module(filename: str, opts: Any, excludes: List[str]) -> bool:
+def is_skipped_module(filename: str, opts: Any, _excludes: Sequence[re.Pattern[str]]) -> bool:
     """Check if we want to skip this module."""
     if not path.exists(filename):
         # skip if the file doesn't exist
         return True
-    elif path.basename(filename).startswith('_') and not opts.includeprivate:
+    if path.basename(filename).startswith('_') and not opts.includeprivate:
         # skip if the module has a "private" name
         return True
-    else:
-        return False
+    return False
 
 
-def walk(rootpath: str, excludes: List[str], opts: Any
-         ) -> Generator[Tuple[str, List[str], List[str]], None, None]:
+def walk(rootpath: str, excludes: Sequence[re.Pattern[str]], opts: Any,
+         ) -> Generator[tuple[str, list[str], list[str]], None, None]:
     """Walk through the directory and list files and subdirectories up."""
     followlinks = getattr(opts, 'followlinks', False)
     includeprivate = getattr(opts, 'includeprivate', False)
@@ -215,7 +230,7 @@ def walk(rootpath: str, excludes: List[str], opts: Any
         # remove hidden ('.') and private ('_') directories, as well as
         # excluded dirs
         if includeprivate:
-            exclude_prefixes: Tuple[str, ...] = ('.',)
+            exclude_prefixes: tuple[str, ...] = ('.',)
         else:
             exclude_prefixes = ('.', '_')
 
@@ -225,17 +240,16 @@ def walk(rootpath: str, excludes: List[str], opts: Any
         yield root, subs, files
 
 
-def has_child_module(rootpath: str, excludes: List[str], opts: Any) -> bool:
-    """Check the given directory contains child modules at least one."""
-    for root, subs, files in walk(rootpath, excludes, opts):
-        if files:
-            return True
-
-    return False
+def has_child_module(rootpath: str, excludes: Sequence[re.Pattern[str]], opts: Any) -> bool:
+    """Check the given directory contains child module/s (at least one)."""
+    return any(
+        files
+        for _root, _subs, files in walk(rootpath, excludes, opts)
+    )
 
 
-def recurse_tree(rootpath: str, excludes: List[str], opts: Any,
-                 user_template_dir: str = None) -> List[str]:
+def recurse_tree(rootpath: str, excludes: Sequence[re.Pattern[str]], opts: Any,
+                 user_template_dir: str | None = None) -> list[str]:
     """
     Look for every file in the directory tree and create the corresponding
     ReST files.
@@ -278,7 +292,8 @@ def recurse_tree(rootpath: str, excludes: List[str], opts: Any,
                     toplevels.append(module_join(root_package, subpackage))
         else:
             # if we are at the root level, we don't require it to be a package
-            assert root == rootpath and root_package is None
+            assert root == rootpath
+            assert root_package is None
             for py_file in files:
                 if not is_skipped_module(path.join(rootpath, py_file), opts, excludes):
                     module = py_file.split('.')[0]
@@ -288,23 +303,20 @@ def recurse_tree(rootpath: str, excludes: List[str], opts: Any,
     return toplevels
 
 
-def is_excluded(root: str, excludes: List[str]) -> bool:
+def is_excluded(root: str, excludes: Sequence[re.Pattern[str]]) -> bool:
     """Check if the directory is in the exclude list.
 
     Note: by having trailing slashes, we avoid common prefix issues, like
           e.g. an exclude "foo" also accidentally excluding "foobar".
     """
-    for exclude in excludes:
-        if fnmatch(root, exclude):
-            return True
-    return False
+    return any(exclude.match(root) for exclude in excludes)
 
 
 def get_parser() -> argparse.ArgumentParser:
     parser = argparse.ArgumentParser(
         usage='%(prog)s [OPTIONS] -o <OUTPUT_PATH> <MODULE_PATH> '
               '[EXCLUDE_PATTERN, ...]',
-        epilog=__('For more information, visit <http://sphinx-doc.org/>.'),
+        epilog=__('For more information, visit <https://www.sphinx-doc.org/>.'),
         description=__("""
 Look recursively in <MODULE_PATH> for Python modules and packages and create
 one reST file with automodule directives per package in the <OUTPUT_PATH>.
@@ -397,13 +409,13 @@ Note: By default this script will not overwrite already created files."""))
     return parser
 
 
-def main(argv: List[str] = sys.argv[1:]) -> int:
+def main(argv: Sequence[str] = (), /) -> int:
     """Parse and check the command line arguments."""
-    sphinx.locale.setlocale(locale.LC_ALL, '')
-    sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
+    locale.setlocale(locale.LC_ALL, '')
+    sphinx.locale.init_console()
 
     parser = get_parser()
-    args = parser.parse_args(argv)
+    args = parser.parse_args(argv or sys.argv[1:])
 
     rootpath = path.abspath(args.module_path)
 
@@ -414,11 +426,14 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
     if args.suffix.startswith('.'):
         args.suffix = args.suffix[1:]
     if not path.isdir(rootpath):
-        print(__('%s is not a directory.') % rootpath, file=sys.stderr)
-        sys.exit(1)
+        logger.error(__('%s is not a directory.'), rootpath)
+        raise SystemExit(1)
     if not args.dryrun:
         ensuredir(args.destdir)
-    excludes = [path.abspath(exclude) for exclude in args.exclude_pattern]
+    excludes = tuple(
+        re.compile(fnmatch.translate(path.abspath(exclude)))
+        for exclude in dict.fromkeys(args.exclude_pattern)
+    )
     modules = recurse_tree(rootpath, excludes, args, args.templatedir)
 
     if args.full:
@@ -474,4 +489,4 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
 
 # So program can be started with "python -m sphinx.apidoc ..."
 if __name__ == "__main__":
-    main()
+    raise SystemExit(main(sys.argv[1:]))
