@@ -1,34 +1,44 @@
-"""
-    sphinx.testing.fixtures
-    ~~~~~~~~~~~~~~~~~~~~~~~
+"""Sphinx test fixtures for pytest"""
 
-    Sphinx test fixtures for pytest
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-import os
+import shutil
 import subprocess
 import sys
 from collections import namedtuple
 from io import StringIO
-from subprocess import PIPE
-from typing import Any, Callable, Dict, Generator, Tuple
+from typing import TYPE_CHECKING, Any, Callable
 
 import pytest
 
-from sphinx.testing import util
 from sphinx.testing.util import SphinxTestApp, SphinxTestAppWrapperForSkipBuilding
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+    from pathlib import Path
+
+DEFAULT_ENABLED_MARKERS = [
+    (
+        'sphinx(builder, testroot=None, freshenv=False, confoverrides=None, tags=None,'
+        ' docutilsconf=None, parallel=0): arguments to initialize the sphinx test application.'
+    ),
+    'test_params(shared_result=...): test parameters.',
+]
+
+
+def pytest_configure(config):
+    """Register custom markers"""
+    for marker in DEFAULT_ENABLED_MARKERS:
+        config.addinivalue_line('markers', marker)
+
 
 @pytest.fixture(scope='session')
-def rootdir() -> str:
+def rootdir() -> str | None:
     return None
 
 
 class SharedResult:
-    cache = {}  # type: Dict[str, Dict[str, str]]
+    cache: dict[str, dict[str, str]] = {}
 
     def store(self, key: str, app_: SphinxTestApp) -> Any:
         if key in self.cache:
@@ -39,7 +49,7 @@ class SharedResult:
         }
         self.cache[key] = data
 
-    def restore(self, key: str) -> Dict[str, StringIO]:
+    def restore(self, key: str) -> dict[str, StringIO]:
         if key not in self.cache:
             return {}
         data = self.cache[key]
@@ -49,38 +59,32 @@ class SharedResult:
         }
 
 
-@pytest.fixture
-def app_params(request: Any, test_params: Dict, shared_result: SharedResult,
-               sphinx_test_tempdir: str, rootdir: str) -> Tuple[Dict, Dict]:
+@pytest.fixture()
+def app_params(request: Any, test_params: dict, shared_result: SharedResult,
+               sphinx_test_tempdir: str, rootdir: str) -> _app_params:
     """
-    parameters that is specified by 'pytest.mark.sphinx' for
+    Parameters that are specified by 'pytest.mark.sphinx' for
     sphinx.application.Sphinx initialization
     """
 
     # ##### process pytest.mark.sphinx
 
-    if hasattr(request.node, 'iter_markers'):  # pytest-3.6.0 or newer
-        markers = request.node.iter_markers("sphinx")
-    else:
-        markers = request.node.get_marker("sphinx")
     pargs = {}
-    kwargs = {}  # type: Dict[str, Any]
+    kwargs: dict[str, Any] = {}
 
-    if markers is not None:
-        # to avoid stacking positional args
-        for info in reversed(list(markers)):
-            for i, a in enumerate(info.args):
-                pargs[i] = a
-            kwargs.update(info.kwargs)
+    # to avoid stacking positional args
+    for info in reversed(list(request.node.iter_markers("sphinx"))):
+        for i, a in enumerate(info.args):
+            pargs[i] = a
+        kwargs.update(info.kwargs)
 
     args = [pargs[i] for i in sorted(pargs.keys())]
 
     # ##### process pytest.mark.test_params
-
     if test_params['shared_result']:
         if 'srcdir' in kwargs:
-            raise pytest.Exception('You can not specify shared_result and '
-                                   'srcdir in same time.')
+            msg = 'You can not specify shared_result and srcdir in same time.'
+            raise pytest.Exception(msg)
         kwargs['srcdir'] = test_params['shared_result']
         restore = shared_result.restore(test_params['shared_result'])
         kwargs.update(restore)
@@ -93,43 +97,43 @@ def app_params(request: Any, test_params: Dict, shared_result: SharedResult,
     # special support for sphinx/tests
     if rootdir and not srcdir.exists():
         testroot_path = rootdir / ('test-' + testroot)
-        testroot_path.copytree(srcdir)
+        shutil.copytree(testroot_path, srcdir)
 
-    return namedtuple('app_params', 'args,kwargs')(args, kwargs)  # type: ignore
+    return _app_params(args, kwargs)
 
 
-@pytest.fixture
-def test_params(request: Any) -> Dict:
+_app_params = namedtuple('_app_params', 'args,kwargs')
+
+
+@pytest.fixture()
+def test_params(request: Any) -> dict:
     """
-    test parameters that is specified by 'pytest.mark.test_params'
+    Test parameters that are specified by 'pytest.mark.test_params'
 
     :param Union[str] shared_result:
        If the value is provided, app._status and app._warning objects will be
        shared in the parametrized test functions and/or test functions that
        have same 'shared_result' value.
-       **NOTE**: You can not specify shared_result and srcdir in same time.
+       **NOTE**: You can not specify both shared_result and srcdir.
     """
-    if hasattr(request.node, 'get_closest_marker'):  # pytest-3.6.0 or newer
-        env = request.node.get_closest_marker('test_params')
-    else:
-        env = request.node.get_marker('test_params')
+    env = request.node.get_closest_marker('test_params')
     kwargs = env.kwargs if env else {}
     result = {
         'shared_result': None,
     }
     result.update(kwargs)
 
-    if (result['shared_result'] and not isinstance(result['shared_result'], str)):
-        raise pytest.Exception('You can only provide a string type of value '
-                               'for "shared_result" ')
+    if result['shared_result'] and not isinstance(result['shared_result'], str):
+        msg = 'You can only provide a string type of value for "shared_result"'
+        raise pytest.Exception(msg)
     return result
 
 
-@pytest.fixture(scope='function')
-def app(test_params: Dict, app_params: Tuple[Dict, Dict], make_app: Callable,
+@pytest.fixture()
+def app(test_params: dict, app_params: tuple[dict, dict], make_app: Callable,
         shared_result: SharedResult) -> Generator[SphinxTestApp, None, None]:
     """
-    provides sphinx.application.Sphinx object
+    Provides the 'sphinx.application.Sphinx' object
     """
     args, kwargs = app_params
     app_ = make_app(*args, **kwargs)
@@ -146,31 +150,29 @@ def app(test_params: Dict, app_params: Tuple[Dict, Dict], make_app: Callable,
         shared_result.store(test_params['shared_result'], app_)
 
 
-@pytest.fixture(scope='function')
+@pytest.fixture()
 def status(app: SphinxTestApp) -> StringIO:
     """
-    compat for testing with previous @with_app decorator
+    Back-compatibility for testing with previous @with_app decorator
     """
     return app._status
 
 
-@pytest.fixture(scope='function')
+@pytest.fixture()
 def warning(app: SphinxTestApp) -> StringIO:
     """
-    compat for testing with previous @with_app decorator
+    Back-compatibility for testing with previous @with_app decorator
     """
     return app._warning
 
 
 @pytest.fixture()
-def make_app(test_params: Dict, monkeypatch: Any) -> Generator[Callable, None, None]:
+def make_app(test_params: dict, monkeypatch: Any) -> Generator[Callable, None, None]:
     """
-    provides make_app function to initialize SphinxTestApp instance.
+    Provides make_app function to initialize SphinxTestApp instance.
     if you want to initialize 'app' in your test function. please use this
     instead of using SphinxTestApp class directory.
     """
-    monkeypatch.setattr('sphinx.application.abspath', lambda x: x)
-
     apps = []
     syspath = sys.path[:]
 
@@ -178,7 +180,7 @@ def make_app(test_params: Dict, monkeypatch: Any) -> Generator[Callable, None, N
         status, warning = StringIO(), StringIO()
         kwargs.setdefault('status', status)
         kwargs.setdefault('warning', warning)
-        app_ = SphinxTestApp(*args, **kwargs)  # type: Any
+        app_: Any = SphinxTestApp(*args, **kwargs)
         apps.append(app_)
         if test_params['shared_result']:
             app_ = SphinxTestAppWrapperForSkipBuilding(app_)
@@ -190,7 +192,7 @@ def make_app(test_params: Dict, monkeypatch: Any) -> Generator[Callable, None, N
         app_.cleanup()
 
 
-@pytest.fixture
+@pytest.fixture()
 def shared_result() -> SharedResult:
     return SharedResult()
 
@@ -200,8 +202,8 @@ def _shared_result_cache() -> None:
     SharedResult.cache.clear()
 
 
-@pytest.fixture
-def if_graphviz_found(app: SphinxTestApp) -> None:
+@pytest.fixture()
+def if_graphviz_found(app: SphinxTestApp) -> None:  # NoQA: PT004
     """
     The test will be skipped when using 'if_graphviz_found' fixture and graphviz
     dot command is not found.
@@ -209,7 +211,7 @@ def if_graphviz_found(app: SphinxTestApp) -> None:
     graphviz_dot = getattr(app.config, 'graphviz_dot', '')
     try:
         if graphviz_dot:
-            subprocess.run([graphviz_dot, '-V'], stdout=PIPE, stderr=PIPE)  # show version
+            subprocess.run([graphviz_dot, '-V'], capture_output=True)  # show version
             return
     except OSError:  # No such file or directory
         pass
@@ -218,21 +220,24 @@ def if_graphviz_found(app: SphinxTestApp) -> None:
 
 
 @pytest.fixture(scope='session')
-def sphinx_test_tempdir(tmpdir_factory: Any) -> "util.path":
-    """
-    temporary directory that wrapped with `path` class.
-    """
-    tmpdir = os.environ.get('SPHINX_TEST_TEMPDIR')  # RemovedInSphinx40Warning
-    if tmpdir is None:
-        tmpdir = tmpdir_factory.getbasetemp()
-
-    return util.path(tmpdir).abspath()
+def sphinx_test_tempdir(tmp_path_factory: Any) -> Path:
+    """Temporary directory."""
+    return tmp_path_factory.getbasetemp()
 
 
-@pytest.fixture
-def tempdir(tmpdir: str) -> "util.path":
+@pytest.fixture()
+def rollback_sysmodules():  # NoQA: PT004
     """
-    temporary directory that wrapped with `path` class.
-    this fixture is for compat with old test implementation.
+    Rollback sys.modules to its value before testing to unload modules
+    during tests.
+
+    For example, used in test_ext_autosummary.py to permit unloading the
+    target module to clear its cache.
     """
-    return util.path(tmpdir)
+    try:
+        sysmodules = list(sys.modules)
+        yield
+    finally:
+        for modname in list(sys.modules):
+            if modname not in sysmodules:
+                sys.modules.pop(modname)
