@@ -1,5 +1,8 @@
 """Helpers for inspecting Python modules."""
 
+from __future__ import annotations
+
+import ast
 import builtins
 import contextlib
 import enum
@@ -8,25 +11,30 @@ import re
 import sys
 import types
 import typing
-from functools import partial, partialmethod
+from collections.abc import Mapping, Sequence
+from functools import cached_property, partial, partialmethod, singledispatchmethod
 from importlib import import_module
-from inspect import Parameter, isclass, ismethod, ismethoddescriptor, ismodule  # NOQA
+from inspect import (  # noqa: F401
+    Parameter,
+    isasyncgenfunction,
+    isclass,
+    ismethod,
+    ismethoddescriptor,
+    ismodule,
+)
 from io import StringIO
-from types import MethodType, ModuleType
-from typing import Any, Callable, Dict, Mapping, Optional, Sequence, Tuple, Type, cast
+from types import (
+    ClassMethodDescriptorType,
+    MethodDescriptorType,
+    MethodType,
+    ModuleType,
+    WrapperDescriptorType,
+)
+from typing import Any, Callable, cast
 
-from sphinx.pycode.ast import ast  # for py36-37
 from sphinx.pycode.ast import unparse as ast_unparse
 from sphinx.util import logging
-from sphinx.util.typing import ForwardRef
-from sphinx.util.typing import stringify as stringify_annotation
-
-if sys.version_info > (3, 7):
-    from types import ClassMethodDescriptorType, MethodDescriptorType, WrapperDescriptorType
-else:
-    ClassMethodDescriptorType = type(object.__init__)
-    MethodDescriptorType = type(str.join)
-    WrapperDescriptorType = type(dict.__dict__['fromkeys'])
+from sphinx.util.typing import ForwardRef, stringify_annotation
 
 logger = logging.getLogger(__name__)
 
@@ -35,18 +43,17 @@ memory_address_re = re.compile(r' at 0x[0-9a-f]{8,16}(?=>)', re.IGNORECASE)
 
 def unwrap(obj: Any) -> Any:
     """Get an original object from wrapped object (wrapped functions)."""
+    if hasattr(obj, '__sphinx_mock__'):
+        # Skip unwrapping mock object to avoid RecursionError
+        return obj
     try:
-        if hasattr(obj, '__sphinx_mock__'):
-            # Skip unwrapping mock object to avoid RecursionError
-            return obj
-        else:
-            return inspect.unwrap(obj)
+        return inspect.unwrap(obj)
     except ValueError:
         # might be a mock object
         return obj
 
 
-def unwrap_all(obj: Any, *, stop: Callable = None) -> Any:
+def unwrap_all(obj: Any, *, stop: Callable | None = None) -> Any:
     """
     Get an original object from wrapped object (unwrapping partials, wrapped
     functions, and other decorators).
@@ -54,19 +61,17 @@ def unwrap_all(obj: Any, *, stop: Callable = None) -> Any:
     while True:
         if stop and stop(obj):
             return obj
-        elif ispartial(obj):
+        if ispartial(obj):
             obj = obj.func
         elif inspect.isroutine(obj) and hasattr(obj, '__wrapped__'):
-            obj = obj.__wrapped__  # type: ignore
-        elif isclassmethod(obj):
-            obj = obj.__func__
-        elif isstaticmethod(obj):
+            obj = obj.__wrapped__
+        elif isclassmethod(obj) or isstaticmethod(obj):
             obj = obj.__func__
         else:
             return obj
 
 
-def getall(obj: Any) -> Optional[Sequence[str]]:
+def getall(obj: Any) -> Sequence[str] | None:
     """Get __all__ attribute of the module as dict.
 
     Return None if given *obj* does not have __all__.
@@ -75,11 +80,9 @@ def getall(obj: Any) -> Optional[Sequence[str]]:
     __all__ = safe_getattr(obj, '__all__', None)
     if __all__ is None:
         return None
-    else:
-        if (isinstance(__all__, (list, tuple)) and all(isinstance(e, str) for e in __all__)):
-            return __all__
-        else:
-            raise ValueError(__all__)
+    if isinstance(__all__, (list, tuple)) and all(isinstance(e, str) for e in __all__):
+        return __all__
+    raise ValueError(__all__)
 
 
 def getannotations(obj: Any) -> Mapping[str, Any]:
@@ -100,7 +103,7 @@ def getglobals(obj: Any) -> Mapping[str, Any]:
         return {}
 
 
-def getmro(obj: Any) -> Tuple[Type, ...]:
+def getmro(obj: Any) -> tuple[type, ...]:
     """Get __mro__ from given *obj* safely."""
     __mro__ = safe_getattr(obj, '__mro__', None)
     if isinstance(__mro__, tuple):
@@ -109,7 +112,7 @@ def getmro(obj: Any) -> Tuple[Type, ...]:
         return ()
 
 
-def getorigbases(obj: Any) -> Optional[Tuple[Any, ...]]:
+def getorigbases(obj: Any) -> tuple[Any, ...] | None:
     """Get __orig_bases__ from *obj* safely."""
     if not inspect.isclass(obj):
         return None
@@ -124,7 +127,7 @@ def getorigbases(obj: Any) -> Optional[Tuple[Any, ...]]:
         return None
 
 
-def getslots(obj: Any) -> Optional[Dict]:
+def getslots(obj: Any) -> dict[str, Any] | None:
     """Get __slots__ attribute of the class as dict.
 
     Return None if gienv *obj* does not have __slots__.
@@ -142,22 +145,18 @@ def getslots(obj: Any) -> Optional[Dict]:
     elif isinstance(__slots__, str):
         return {__slots__: None}
     elif isinstance(__slots__, (list, tuple)):
-        return {e: None for e in __slots__}
+        return dict.fromkeys(__slots__)
     else:
         raise ValueError
 
 
 def isNewType(obj: Any) -> bool:
     """Check the if object is a kind of NewType."""
-    if sys.version_info >= (3, 10):
+    if sys.version_info[:2] >= (3, 10):
         return isinstance(obj, typing.NewType)
-    else:
-        __module__ = safe_getattr(obj, '__module__', None)
-        __qualname__ = safe_getattr(obj, '__qualname__', None)
-        if __module__ == 'typing' and __qualname__ == 'NewType.<locals>.new_type':
-            return True
-        else:
-            return False
+    __module__ = safe_getattr(obj, '__module__', None)
+    __qualname__ = safe_getattr(obj, '__qualname__', None)
+    return __module__ == 'typing' and __qualname__ == 'NewType.<locals>.new_type'
 
 
 def isenumclass(x: Any) -> bool:
@@ -186,13 +185,13 @@ def ispartial(obj: Any) -> bool:
     return isinstance(obj, (partial, partialmethod))
 
 
-def isclassmethod(obj: Any, cls: Any = None, name: str = None) -> bool:
+def isclassmethod(obj: Any, cls: Any = None, name: str | None = None) -> bool:
     """Check if the object is classmethod."""
     if isinstance(obj, classmethod):
         return True
-    elif inspect.ismethod(obj) and obj.__self__ is not None and isclass(obj.__self__):
+    if inspect.ismethod(obj) and obj.__self__ is not None and isclass(obj.__self__):
         return True
-    elif cls and name:
+    if cls and name:
         placeholder = object()
         for basecls in getmro(cls):
             meth = basecls.__dict__.get(name, placeholder)
@@ -202,31 +201,27 @@ def isclassmethod(obj: Any, cls: Any = None, name: str = None) -> bool:
     return False
 
 
-def isstaticmethod(obj: Any, cls: Any = None, name: str = None) -> bool:
+def isstaticmethod(obj: Any, cls: Any = None, name: str | None = None) -> bool:
     """Check if the object is staticmethod."""
     if isinstance(obj, staticmethod):
         return True
-    elif cls and name:
+    if cls and name:
         # trace __mro__ if the method is defined in parent class
         #
         # .. note:: This only works well with new style classes.
         for basecls in getattr(cls, '__mro__', [cls]):
             meth = basecls.__dict__.get(name)
             if meth:
-                if isinstance(meth, staticmethod):
-                    return True
-                else:
-                    return False
-
+                return isinstance(meth, staticmethod)
     return False
 
 
 def isdescriptor(x: Any) -> bool:
     """Check if the object is some kind of descriptor."""
-    for item in '__get__', '__set__', '__delete__':
-        if callable(safe_getattr(x, item, None)):
-            return True
-    return False
+    return any(
+        callable(safe_getattr(x, item, None))
+        for item in ['__get__', '__set__', '__delete__']
+    )
 
 
 def isabstractmethod(obj: Any) -> bool:
@@ -252,65 +247,56 @@ def isattributedescriptor(obj: Any) -> bool:
     if inspect.isdatadescriptor(obj):
         # data descriptor is kind of attribute
         return True
-    elif isdescriptor(obj):
+    if isdescriptor(obj):
         # non data descriptor
         unwrapped = unwrap(obj)
         if isfunction(unwrapped) or isbuiltin(unwrapped) or inspect.ismethod(unwrapped):
             # attribute must not be either function, builtin and method
             return False
-        elif is_cython_function_or_method(unwrapped):
+        if is_cython_function_or_method(unwrapped):
             # attribute must not be either function and method (for cython)
             return False
-        elif inspect.isclass(unwrapped):
+        if inspect.isclass(unwrapped):
             # attribute must not be a class
             return False
-        elif isinstance(unwrapped, (ClassMethodDescriptorType,
-                                    MethodDescriptorType,
-                                    WrapperDescriptorType)):
+        if isinstance(unwrapped, (ClassMethodDescriptorType,
+                                  MethodDescriptorType,
+                                  WrapperDescriptorType)):
             # attribute must not be a method descriptor
             return False
-        elif type(unwrapped).__name__ == "instancemethod":
+        if type(unwrapped).__name__ == "instancemethod":
             # attribute must not be an instancemethod (C-API)
             return False
-        else:
-            return True
-    else:
-        return False
+        return True
+    return False
 
 
 def is_singledispatch_function(obj: Any) -> bool:
     """Check if the object is singledispatch function."""
-    if (inspect.isfunction(obj) and
+    return (inspect.isfunction(obj) and
             hasattr(obj, 'dispatch') and
             hasattr(obj, 'register') and
-            obj.dispatch.__module__ == 'functools'):  # type: ignore
-        return True
-    else:
-        return False
+            obj.dispatch.__module__ == 'functools')
 
 
 def is_singledispatch_method(obj: Any) -> bool:
     """Check if the object is singledispatch method."""
-    try:
-        from functools import singledispatchmethod  # type: ignore
-        return isinstance(obj, singledispatchmethod)
-    except ImportError:  # py36-37
-        return False
+    return isinstance(obj, singledispatchmethod)
 
 
 def isfunction(obj: Any) -> bool:
     """Check if the object is function."""
-    return inspect.isfunction(unwrap_all(obj))
+    return inspect.isfunction(unpartial(obj))
 
 
 def isbuiltin(obj: Any) -> bool:
-    """Check if the object is builtin."""
-    return inspect.isbuiltin(unwrap_all(obj))
+    """Check if the object is function."""
+    return inspect.isbuiltin(unpartial(obj))
 
 
 def isroutine(obj: Any) -> bool:
     """Check is any kind of function or method."""
-    return inspect.isroutine(unwrap_all(obj))
+    return inspect.isroutine(unpartial(obj))
 
 
 def iscoroutinefunction(obj: Any) -> bool:
@@ -321,53 +307,21 @@ def iscoroutinefunction(obj: Any) -> bool:
             # staticmethod, classmethod and partial method are not a wrapped coroutine-function
             # Note: Since 3.10, staticmethod and classmethod becomes a kind of wrappers
             return False
-        elif hasattr(obj, '__wrapped__'):
-            return True
-        else:
-            return False
+        return hasattr(obj, '__wrapped__')
 
     obj = unwrap_all(obj, stop=iswrappedcoroutine)
-    if hasattr(obj, '__code__') and inspect.iscoroutinefunction(obj):
-        # check obj.__code__ because iscoroutinefunction() crashes for custom method-like
-        # objects (see https://github.com/sphinx-doc/sphinx/issues/6605)
-        return True
-    else:
-        return False
-
-
-def isasyncgenfunction(obj: Any) -> bool:
-    """Check if the object is async-gen function."""
-    if hasattr(obj, '__code__') and inspect.isasyncgenfunction(obj):
-        # check obj.__code__ because isasyncgenfunction() crashes for custom method-like
-        # objects on python3.7 (see https://github.com/sphinx-doc/sphinx/issues/9838)
-        return True
-    else:
-        return False
+    return inspect.iscoroutinefunction(obj)
 
 
 def isproperty(obj: Any) -> bool:
     """Check if the object is property."""
-    if sys.version_info >= (3, 8):
-        from functools import cached_property  # cached_property is available since py3.8
-        if isinstance(obj, cached_property):
-            return True
-
-    return isinstance(obj, property)
+    return isinstance(obj, (property, cached_property))
 
 
 def isgenericalias(obj: Any) -> bool:
     """Check if the object is GenericAlias."""
-    if (hasattr(typing, '_GenericAlias') and  # only for py37+
-            isinstance(obj, typing._GenericAlias)):  # type: ignore
-        return True
-    elif (hasattr(types, 'GenericAlias') and  # only for py39+
-          isinstance(obj, types.GenericAlias)):  # type: ignore
-        return True
-    elif (hasattr(typing, '_SpecialGenericAlias') and  # for py39+
-            isinstance(obj, typing._SpecialGenericAlias)):  # type: ignore
-        return True
-    else:
-        return False
+    return isinstance(
+        obj, (types.GenericAlias, typing._BaseGenericAlias))  # type: ignore[attr-defined]
 
 
 def safe_getattr(obj: Any, name: str, *defargs: Any) -> Any:
@@ -392,38 +346,64 @@ def safe_getattr(obj: Any, name: str, *defargs: Any) -> Any:
         raise AttributeError(name) from exc
 
 
-def object_description(object: Any) -> str:
-    """A repr() implementation that returns text safe to use in reST context."""
-    if isinstance(object, dict):
+def object_description(obj: Any, *, _seen: frozenset = frozenset()) -> str:
+    """A repr() implementation that returns text safe to use in reST context.
+
+    Maintains a set of 'seen' object IDs to detect and avoid infinite recursion.
+    """
+    seen = _seen
+    if isinstance(obj, dict):
+        if id(obj) in seen:
+            return 'dict(...)'
+        seen |= {id(obj)}
         try:
-            sorted_keys = sorted(object)
-        except Exception:
-            pass  # Cannot sort dict keys, fall back to generic repr
-        else:
-            items = ("%s: %s" %
-                     (object_description(key), object_description(object[key]))
-                     for key in sorted_keys)
-            return "{%s}" % ", ".join(items)
-    elif isinstance(object, set):
-        try:
-            sorted_values = sorted(object)
+            sorted_keys = sorted(obj)
         except TypeError:
-            pass  # Cannot sort set values, fall back to generic repr
-        else:
-            return "{%s}" % ", ".join(object_description(x) for x in sorted_values)
-    elif isinstance(object, frozenset):
+            # Cannot sort dict keys, fall back to using descriptions as a sort key
+            sorted_keys = sorted(obj, key=lambda k: object_description(k, _seen=seen))
+
+        items = ((object_description(key, _seen=seen),
+                  object_description(obj[key], _seen=seen)) for key in sorted_keys)
+        return '{%s}' % ', '.join(f'{key}: {value}' for (key, value) in items)
+    elif isinstance(obj, set):
+        if id(obj) in seen:
+            return 'set(...)'
+        seen |= {id(obj)}
         try:
-            sorted_values = sorted(object)
+            sorted_values = sorted(obj)
         except TypeError:
-            pass  # Cannot sort frozenset values, fall back to generic repr
-        else:
-            return "frozenset({%s})" % ", ".join(object_description(x)
-                                                 for x in sorted_values)
-    elif isinstance(object, enum.Enum):
-        return "%s.%s" % (object.__class__.__name__, object.name)
+            # Cannot sort set values, fall back to using descriptions as a sort key
+            sorted_values = sorted(obj, key=lambda x: object_description(x, _seen=seen))
+        return '{%s}' % ', '.join(object_description(x, _seen=seen) for x in sorted_values)
+    elif isinstance(obj, frozenset):
+        if id(obj) in seen:
+            return 'frozenset(...)'
+        seen |= {id(obj)}
+        try:
+            sorted_values = sorted(obj)
+        except TypeError:
+            # Cannot sort frozenset values, fall back to using descriptions as a sort key
+            sorted_values = sorted(obj, key=lambda x: object_description(x, _seen=seen))
+        return 'frozenset({%s})' % ', '.join(object_description(x, _seen=seen)
+                                             for x in sorted_values)
+    elif isinstance(obj, enum.Enum):
+        return f'{obj.__class__.__name__}.{obj.name}'
+    elif isinstance(obj, tuple):
+        if id(obj) in seen:
+            return 'tuple(...)'
+        seen |= frozenset([id(obj)])
+        return '(%s%s)' % (
+            ', '.join(object_description(x, _seen=seen) for x in obj),
+            ',' * (len(obj) == 1),
+        )
+    elif isinstance(obj, list):
+        if id(obj) in seen:
+            return 'list(...)'
+        seen |= {id(obj)}
+        return '[%s]' % ', '.join(object_description(x, _seen=seen) for x in obj)
 
     try:
-        s = repr(object)
+        s = repr(obj)
     except Exception as exc:
         raise ValueError from exc
     # Strip non-deterministic memory addresses such as
@@ -493,11 +473,11 @@ class TypeAliasForwardRef:
 class TypeAliasModule:
     """Pseudo module class for autodoc_type_aliases."""
 
-    def __init__(self, modname: str, mapping: Dict[str, str]) -> None:
+    def __init__(self, modname: str, mapping: dict[str, str]) -> None:
         self.__modname = modname
         self.__mapping = mapping
 
-        self.__module: Optional[ModuleType] = None
+        self.__module: ModuleType | None = None
 
     def __getattr__(self, name: str) -> Any:
         fullname = '.'.join(filter(None, [self.__modname, name]))
@@ -523,13 +503,13 @@ class TypeAliasModule:
                     return getattr(self.__module, name)
 
 
-class TypeAliasNamespace(Dict[str, Any]):
+class TypeAliasNamespace(dict[str, Any]):
     """Pseudo namespace class for autodoc_type_aliases.
 
     This enables to look up nested modules and classes like `mod1.mod2.Class`.
     """
 
-    def __init__(self, mapping: Dict[str, str]) -> None:
+    def __init__(self, mapping: dict[str, str]) -> None:
         self.__mapping = mapping
 
     def __getitem__(self, key: str) -> Any:
@@ -557,33 +537,25 @@ def _should_unwrap(subject: Callable) -> bool:
     return False
 
 
-def signature(subject: Callable, bound_method: bool = False, type_aliases: Dict = {}
+def signature(subject: Callable, bound_method: bool = False, type_aliases: dict | None = None,
               ) -> inspect.Signature:
     """Return a Signature object for the given *subject*.
 
     :param bound_method: Specify *subject* is a bound method or not
     """
+    if type_aliases is None:
+        type_aliases = {}
 
     try:
-        try:
-            if _should_unwrap(subject):
-                signature = inspect.signature(subject)
-            else:
-                signature = inspect.signature(subject, follow_wrapped=True)
-        except ValueError:
-            # follow built-in wrappers up (ex. functools.lru_cache)
+        if _should_unwrap(subject):
             signature = inspect.signature(subject)
-        parameters = list(signature.parameters.values())
-        return_annotation = signature.return_annotation
-    except IndexError:
-        # Until python 3.6.4, cpython has been crashed on inspection for
-        # partialmethods not having any arguments.
-        # https://bugs.python.org/issue33009
-        if hasattr(subject, '_partialmethod'):
-            parameters = []
-            return_annotation = Parameter.empty
         else:
-            raise
+            signature = inspect.signature(subject, follow_wrapped=True)
+    except ValueError:
+        # follow built-in wrappers up (ex. functools.lru_cache)
+        signature = inspect.signature(subject)
+    parameters = list(signature.parameters.values())
+    return_annotation = signature.return_annotation
 
     try:
         # Resolve annotations using ``get_type_hints()`` and type_aliases.
@@ -623,17 +595,15 @@ def signature(subject: Callable, bound_method: bool = False, type_aliases: Dict
                              __validate_parameters__=False)
 
 
-def evaluate_signature(sig: inspect.Signature, globalns: Dict = None, localns: Dict = None
+def evaluate_signature(sig: inspect.Signature, globalns: dict | None = None,
+                       localns: dict | None = None,
                        ) -> inspect.Signature:
     """Evaluate unresolved type annotations in a signature object."""
-    def evaluate_forwardref(ref: ForwardRef, globalns: Dict, localns: Dict) -> Any:
+    def evaluate_forwardref(ref: ForwardRef, globalns: dict, localns: dict) -> Any:
         """Evaluate a forward reference."""
-        if sys.version_info > (3, 9):
-            return ref._evaluate(globalns, localns, frozenset())
-        else:
-            return ref._evaluate(globalns, localns)
+        return ref._evaluate(globalns, localns, frozenset())
 
-    def evaluate(annotation: Any, globalns: Dict, localns: Dict) -> Any:
+    def evaluate(annotation: Any, globalns: dict, localns: dict) -> Any:
         """Evaluate unresolved type annotation."""
         try:
             if isinstance(annotation, str):
@@ -722,20 +692,21 @@ def stringify_signature(sig: inspect.Signature, show_annotation: bool = True,
         # PEP-570: Separator for Positional Only Parameter: /
         args.append('/')
 
+    concatenated_args = ', '.join(args)
     if (sig.return_annotation is Parameter.empty or
             show_annotation is False or
             show_return_annotation is False):
-        return '(%s)' % ', '.join(args)
+        return f'({concatenated_args})'
     else:
         annotation = stringify_annotation(sig.return_annotation, mode)
-        return '(%s) -> %s' % (', '.join(args), annotation)
+        return f'({concatenated_args}) -> {annotation}'
 
 
 def signature_from_str(signature: str) -> inspect.Signature:
     """Create a Signature object from string."""
     code = 'def func' + signature + ': pass'
     module = ast.parse(code)
-    function = cast(ast.FunctionDef, module.body[0])  # type: ignore
+    function = cast(ast.FunctionDef, module.body[0])
 
     return signature_from_ast(function, code)
 
@@ -746,21 +717,22 @@ def signature_from_ast(node: ast.FunctionDef, code: str = '') -> inspect.Signatu
     defaults = list(args.defaults)
     params = []
     if hasattr(args, "posonlyargs"):
-        posonlyargs = len(args.posonlyargs)  # type: ignore
+        posonlyargs = len(args.posonlyargs)
         positionals = posonlyargs + len(args.args)
     else:
         posonlyargs = 0
         positionals = len(args.args)
 
     for _ in range(len(defaults), positionals):
-        defaults.insert(0, Parameter.empty)  # type: ignore
+        defaults.insert(0, Parameter.empty)  # type: ignore[arg-type]
 
     if hasattr(args, "posonlyargs"):
-        for i, arg in enumerate(args.posonlyargs):  # type: ignore
+        for i, arg in enumerate(args.posonlyargs):
             if defaults[i] is Parameter.empty:
                 default = Parameter.empty
             else:
-                default = DefaultValue(ast_unparse(defaults[i], code))  # type: ignore
+                default = DefaultValue(
+                    ast_unparse(defaults[i], code))  # type: ignore[assignment]
 
             annotation = ast_unparse(arg.annotation, code) or Parameter.empty
             params.append(Parameter(arg.arg, Parameter.POSITIONAL_ONLY,
@@ -770,7 +742,9 @@ def signature_from_ast(node: ast.FunctionDef, code: str = '') -> inspect.Signatu
         if defaults[i + posonlyargs] is Parameter.empty:
             default = Parameter.empty
         else:
-            default = DefaultValue(ast_unparse(defaults[i + posonlyargs], code))  # type: ignore  # NOQA
+            default = DefaultValue(
+                ast_unparse(defaults[i + posonlyargs], code),  # type: ignore[assignment]
+            )
 
         annotation = ast_unparse(arg.annotation, code) or Parameter.empty
         params.append(Parameter(arg.arg, Parameter.POSITIONAL_OR_KEYWORD,
@@ -785,7 +759,8 @@ def signature_from_ast(node: ast.FunctionDef, code: str = '') -> inspect.Signatu
         if args.kw_defaults[i] is None:
             default = Parameter.empty
         else:
-            default = DefaultValue(ast_unparse(args.kw_defaults[i], code))  # type: ignore  # NOQA
+            default = DefaultValue(
+                ast_unparse(args.kw_defaults[i], code))  # type: ignore[arg-type,assignment]
         annotation = ast_unparse(arg.annotation, code) or Parameter.empty
         params.append(Parameter(arg.arg, Parameter.KEYWORD_ONLY, default=default,
                                 annotation=annotation))
@@ -800,8 +775,13 @@ def signature_from_ast(node: ast.FunctionDef, code: str = '') -> inspect.Signatu
     return inspect.Signature(params, return_annotation=return_annotation)
 
 
-def getdoc(obj: Any, attrgetter: Callable = safe_getattr,
-           allow_inherited: bool = False, cls: Any = None, name: str = None) -> str:
+def getdoc(
+    obj: Any,
+    attrgetter: Callable = safe_getattr,
+    allow_inherited: bool = False,
+    cls: Any = None,
+    name: str | None = None,
+) -> str | None:
     """Get the docstring for the object.
 
     This tries to obtain the docstring for some kind of objects additionally:
@@ -810,7 +790,7 @@ def getdoc(obj: Any, attrgetter: Callable = safe_getattr,
     * inherited docstring
     * inherited decorated methods
     """
-    def getdoc_internal(obj: Any, attrgetter: Callable = safe_getattr) -> Optional[str]:
+    def getdoc_internal(obj: Any, attrgetter: Callable = safe_getattr) -> str | None:
         doc = attrgetter(obj, '__doc__', None)
         if isinstance(doc, str):
             return doc
@@ -821,7 +801,7 @@ def getdoc(obj: Any, attrgetter: Callable = safe_getattr,
         for basecls in getmro(cls):
             meth = basecls.__dict__.get(name)
             if meth and hasattr(meth, '__func__'):
-                doc = getdoc(meth.__func__)
+                doc: str | None = getdoc(meth.__func__)
                 if doc is not None or not allow_inherited:
                     return doc
 
