@@ -1,51 +1,42 @@
-"""
-    sphinx.util.inventory
-    ~~~~~~~~~~~~~~~~~~~~~
+"""Inventory utility functions for Sphinx."""
+from __future__ import annotations
 
-    Inventory utility functions for Sphinx.
-
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
 import os
 import re
 import zlib
+from typing import IO, TYPE_CHECKING, Callable
 
 from sphinx.util import logging
 
-if False:
-    # For type annotation
-    from typing import Callable, IO, Iterator  # NOQA
-    from sphinx.builders import Builder  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
-    from sphinx.util.typing import Inventory  # NOQA
-
-
 BUFSIZE = 16 * 1024
 logger = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+    from collections.abc import Iterator
+
+    from sphinx.builders import Builder
+    from sphinx.environment import BuildEnvironment
+    from sphinx.util.typing import Inventory, InventoryItem
+
 
 class InventoryFileReader:
-    """A file reader for inventory file.
+    """A file reader for an inventory file.
 
     This reader supports mixture of texts and compressed texts.
     """
 
-    def __init__(self, stream):
-        # type: (IO) -> None
+    def __init__(self, stream: IO) -> None:
         self.stream = stream
         self.buffer = b''
         self.eof = False
 
-    def read_buffer(self):
-        # type: () -> None
+    def read_buffer(self) -> None:
         chunk = self.stream.read(BUFSIZE)
         if chunk == b'':
             self.eof = True
         self.buffer += chunk
 
-    def readline(self):
-        # type: () -> str
+    def readline(self) -> str:
         pos = self.buffer.find(b'\n')
         if pos != -1:
             line = self.buffer[:pos].decode()
@@ -59,15 +50,13 @@ class InventoryFileReader:
 
         return line
 
-    def readlines(self):
-        # type: () -> Iterator[str]
+    def readlines(self) -> Iterator[str]:
         while not self.eof:
             line = self.readline()
             if line:
                 yield line
 
-    def read_compressed_chunks(self):
-        # type: () -> Iterator[bytes]
+    def read_compressed_chunks(self) -> Iterator[bytes]:
         decompressor = zlib.decompressobj()
         while not self.eof:
             self.read_buffer()
@@ -75,8 +64,7 @@ class InventoryFileReader:
             self.buffer = b''
         yield decompressor.flush()
 
-    def read_compressed_lines(self):
-        # type: () -> Iterator[str]
+    def read_compressed_lines(self) -> Iterator[str]:
         buf = b''
         for chunk in self.read_compressed_chunks():
             buf += chunk
@@ -89,8 +77,7 @@ class InventoryFileReader:
 
 class InventoryFile:
     @classmethod
-    def load(cls, stream, uri, joinfunc):
-        # type: (IO, str, Callable) -> Inventory
+    def load(cls, stream: IO, uri: str, joinfunc: Callable) -> Inventory:
         reader = InventoryFileReader(stream)
         line = reader.readline().rstrip()
         if line == '# Sphinx inventory version 1':
@@ -101,9 +88,8 @@ class InventoryFile:
             raise ValueError('invalid inventory header: %s' % line)
 
     @classmethod
-    def load_v1(cls, stream, uri, join):
-        # type: (InventoryFileReader, str, Callable) -> Inventory
-        invdata = {}  # type: Inventory
+    def load_v1(cls, stream: InventoryFileReader, uri: str, join: Callable) -> Inventory:
+        invdata: Inventory = {}
         projname = stream.readline().rstrip()[11:]
         version = stream.readline().rstrip()[11:]
         for line in stream.readlines():
@@ -120,9 +106,8 @@ class InventoryFile:
         return invdata
 
     @classmethod
-    def load_v2(cls, stream, uri, join):
-        # type: (InventoryFileReader, str, Callable) -> Inventory
-        invdata = {}  # type: Inventory
+    def load_v2(cls, stream: InventoryFileReader, uri: str, join: Callable) -> Inventory:
+        invdata: Inventory = {}
         projname = stream.readline().rstrip()[11:]
         version = stream.readline().rstrip()[11:]
         line = stream.readline()
@@ -131,11 +116,16 @@ class InventoryFile:
 
         for line in stream.read_compressed_lines():
             # be careful to handle names with embedded spaces correctly
-            m = re.match(r'(?x)(.+?)\s+(\S*:\S*)\s+(-?\d+)\s+(\S+)\s+(.*)',
-                         line.rstrip())
+            m = re.match(r'(.+?)\s+(\S+)\s+(-?\d+)\s+?(\S*)\s+(.*)',
+                         line.rstrip(), flags=re.VERBOSE)
             if not m:
                 continue
             name, type, prio, location, dispname = m.groups()
+            if ':' not in type:
+                # wrong type value. type should be in the form of "{domain}:{objtype}"
+                #
+                # Note: To avoid the regex DoS, this is implemented in python (refs: #8175)
+                continue
             if type == 'py:module' and type in invdata and name in invdata[type]:
                 # due to a bug in 1.1 and below,
                 # two inventory entries are created
@@ -145,15 +135,13 @@ class InventoryFile:
             if location.endswith('$'):
                 location = location[:-1] + name
             location = join(uri, location)
-            invdata.setdefault(type, {})[name] = (projname, version,
-                                                  location, dispname)
+            inv_item: InventoryItem = projname, version, location, dispname
+            invdata.setdefault(type, {})[name] = inv_item
         return invdata
 
     @classmethod
-    def dump(cls, filename, env, builder):
-        # type: (str, BuildEnvironment, Builder) -> None
-        def escape(string):
-            # type: (str) -> str
+    def dump(cls, filename: str, env: BuildEnvironment, builder: Builder) -> None:
+        def escape(string: str) -> str:
             return re.sub("\\s+", " ", string)
 
         with open(os.path.join(filename), 'wb') as f:
