@@ -1,21 +1,22 @@
 """Operating system-related utility functions for Sphinx."""
 
+from __future__ import annotations
+
 import contextlib
 import filecmp
 import os
 import re
 import shutil
 import sys
+import unicodedata
 from io import StringIO
 from os import path
-from typing import Any, Generator, Iterator, List, Optional, Type
+from typing import TYPE_CHECKING, Any
 
-try:
-    # for ALT Linux (#6712)
-    from sphinx.testing.path import path as Path
-except ImportError:
-    Path = None  # type: ignore
+from sphinx.deprecation import _deprecation_warning
 
+if TYPE_CHECKING:
+    from collections.abc import Iterator
 
 # SEP separates path elements in the canonical file names
 #
@@ -25,13 +26,19 @@ except ImportError:
 SEP = "/"
 
 
-def os_path(canonicalpath: str) -> str:
-    return canonicalpath.replace(SEP, path.sep)
+def os_path(canonical_path: str, /) -> str:
+    return canonical_path.replace(SEP, path.sep)
 
 
-def canon_path(nativepath: str) -> str:
+def canon_path(native_path: str | os.PathLike[str], /) -> str:
     """Return path in OS-independent form"""
-    return nativepath.replace(path.sep, SEP)
+    return os.fspath(native_path).replace(path.sep, SEP)
+
+
+def path_stabilize(filepath: str | os.PathLike[str], /) -> str:
+    "Normalize path separator and unicode string"
+    new_path = canon_path(filepath)
+    return unicodedata.normalize('NFC', new_path)
 
 
 def relative_uri(base: str, to: str) -> str:
@@ -57,40 +64,36 @@ def relative_uri(base: str, to: str) -> str:
     return ('..' + SEP) * (len(b2) - 1) + SEP.join(t2)
 
 
-def ensuredir(path: str) -> None:
+def ensuredir(file: str | os.PathLike[str]) -> None:
     """Ensure that a path exists."""
-    os.makedirs(path, exist_ok=True)
+    os.makedirs(file, exist_ok=True)
 
 
-def mtimes_of_files(dirnames: List[str], suffix: str) -> Iterator[float]:
+def mtimes_of_files(dirnames: list[str], suffix: str) -> Iterator[float]:
     for dirname in dirnames:
         for root, _dirs, files in os.walk(dirname):
             for sfile in files:
                 if sfile.endswith(suffix):
-                    try:
+                    with contextlib.suppress(OSError):
                         yield path.getmtime(path.join(root, sfile))
-                    except OSError:
-                        pass
 
 
-def copytimes(source: str, dest: str) -> None:
+def copytimes(source: str | os.PathLike[str], dest: str | os.PathLike[str]) -> None:
     """Copy a file's modification times."""
     st = os.stat(source)
     if hasattr(os, 'utime'):
         os.utime(dest, (st.st_atime, st.st_mtime))
 
 
-def copyfile(source: str, dest: str) -> None:
+def copyfile(source: str | os.PathLike[str], dest: str | os.PathLike[str]) -> None:
     """Copy a file and its modification times, if possible.
 
     Note: ``copyfile`` skips copying if the file has not been changed"""
     if not path.exists(dest) or not filecmp.cmp(source, dest):
         shutil.copyfile(source, dest)
-        try:
+        with contextlib.suppress(OSError):
             # don't do full copystat because the source may be read-only
             copytimes(source, dest)
-        except OSError:
-            pass
 
 
 no_fn_re = re.compile(r'[^a-zA-Z0-9_-]')
@@ -105,7 +108,8 @@ def make_filename_from_project(project: str) -> str:
     return make_filename(project_suffix_re.sub('', project)).lower()
 
 
-def relpath(path: str, start: str = os.curdir) -> str:
+def relpath(path: str | os.PathLike[str],
+            start: str | os.PathLike[str] | None = os.curdir) -> str:
     """Return a relative filepath to *path* either from the current directory or
     from an optional *start* directory.
 
@@ -115,36 +119,36 @@ def relpath(path: str, start: str = os.curdir) -> str:
     try:
         return os.path.relpath(path, start)
     except ValueError:
-        return path
+        return str(path)
 
 
 safe_relpath = relpath  # for compatibility
 fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
 
 
-def abspath(pathdir: str) -> str:
-    if Path is not None and isinstance(pathdir, Path):
-        return pathdir.abspath()
-    else:
-        pathdir = path.abspath(pathdir)
-        if isinstance(pathdir, bytes):
-            try:
-                pathdir = pathdir.decode(fs_encoding)
-            except UnicodeDecodeError as exc:
-                raise UnicodeDecodeError('multibyte filename not supported on '
-                                         'this filesystem encoding '
-                                         '(%r)' % fs_encoding) from exc
-        return pathdir
+abspath = path.abspath
+
+
+class _chdir:
+    """Remove this fall-back once support for Python 3.10 is removed."""
+    def __init__(self, target_dir: str, /):
+        self.path = target_dir
+        self._dirs: list[str] = []
+
+    def __enter__(self):
+        self._dirs.append(os.getcwd())
+        os.chdir(self.path)
+
+    def __exit__(self, _exc_type, _exc_value, _traceback, /):
+        os.chdir(self._dirs.pop())
 
 
 @contextlib.contextmanager
-def cd(target_dir: str) -> Generator[None, None, None]:
-    cwd = os.getcwd()
-    try:
-        os.chdir(target_dir)
+def cd(target_dir: str) -> Iterator[None]:
+    if sys.version_info[:2] >= (3, 11):
+        _deprecation_warning(__name__, 'cd', 'contextlib.chdir', remove=(8, 0))
+    with _chdir(target_dir):
         yield
-    finally:
-        os.chdir(cwd)
 
 
 class FileAvoidWrite:
@@ -161,7 +165,7 @@ class FileAvoidWrite:
     """
     def __init__(self, path: str) -> None:
         self._path = path
-        self._io: Optional[StringIO] = None
+        self._io: StringIO | None = None
 
     def write(self, data: str) -> None:
         if not self._io:
@@ -171,7 +175,8 @@ class FileAvoidWrite:
     def close(self) -> None:
         """Stop accepting writes and write file, if needed."""
         if not self._io:
-            raise Exception('FileAvoidWrite does not support empty files.')
+            msg = 'FileAvoidWrite does not support empty files.'
+            raise Exception(msg)
 
         buf = self.getvalue()
         self._io.close()
@@ -187,18 +192,20 @@ class FileAvoidWrite:
         with open(self._path, 'w', encoding='utf-8') as f:
             f.write(buf)
 
-    def __enter__(self) -> "FileAvoidWrite":
+    def __enter__(self) -> FileAvoidWrite:
         return self
 
-    def __exit__(self, exc_type: Type[Exception], exc_value: Exception, traceback: Any) -> bool:  # NOQA
+    def __exit__(
+        self, exc_type: type[Exception], exc_value: Exception, traceback: Any,
+    ) -> bool:
         self.close()
         return True
 
     def __getattr__(self, name: str) -> Any:
         # Proxy to _io instance.
         if not self._io:
-            raise Exception('Must write to FileAvoidWrite before other '
-                            'methods can be used')
+            msg = 'Must write to FileAvoidWrite before other methods can be used'
+            raise Exception(msg)
 
         return getattr(self._io, name)
 
