@@ -1,20 +1,13 @@
-"""
-    sphinx.util.template
-    ~~~~~~~~~~~~~~~~~~~~
+"""Templates utility functions for Sphinx."""
 
-    Templates utility functions for Sphinx.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import os
 from functools import partial
 from os import path
-from typing import Callable, Dict, List, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable
 
 from jinja2 import TemplateNotFound
-from jinja2.environment import Environment
 from jinja2.loaders import BaseLoader
 from jinja2.sandbox import SandboxedEnvironment
 
@@ -23,23 +16,28 @@ from sphinx.jinja2glue import SphinxFileSystemLoader
 from sphinx.locale import get_translator
 from sphinx.util import rst, texescape
 
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+
+    from jinja2.environment import Environment
+
 
 class BaseRenderer:
-    def __init__(self, loader: BaseLoader = None) -> None:
+    def __init__(self, loader: BaseLoader | None = None) -> None:
         self.env = SandboxedEnvironment(loader=loader, extensions=['jinja2.ext.i18n'])
         self.env.filters['repr'] = repr
         self.env.install_gettext_translations(get_translator())
 
-    def render(self, template_name: str, context: Dict) -> str:
+    def render(self, template_name: str, context: dict[str, Any]) -> str:
         return self.env.get_template(template_name).render(context)
 
-    def render_string(self, source: str, context: Dict) -> str:
+    def render_string(self, source: str, context: dict[str, Any]) -> str:
         return self.env.from_string(source).render(context)
 
 
 class FileRenderer(BaseRenderer):
-    def __init__(self, search_path: Union[str, List[str]]) -> None:
-        if isinstance(search_path, str):
+    def __init__(self, search_path: Sequence[str | os.PathLike[str]]) -> None:
+        if isinstance(search_path, (str, os.PathLike)):
             search_path = [search_path]
         else:
             # filter "None" paths
@@ -49,27 +47,28 @@ class FileRenderer(BaseRenderer):
         super().__init__(loader)
 
     @classmethod
-    def render_from_file(cls, filename: str, context: Dict) -> str:
+    def render_from_file(cls, filename: str, context: dict[str, Any]) -> str:
         dirname = os.path.dirname(filename)
         basename = os.path.basename(filename)
         return cls(dirname).render(basename, context)
 
 
 class SphinxRenderer(FileRenderer):
-    def __init__(self, template_path: Union[str, List[str]] = None) -> None:
+    def __init__(self, template_path: Sequence[str | os.PathLike[str]] | None = None) -> None:
         if template_path is None:
             template_path = os.path.join(package_dir, 'templates')
         super().__init__(template_path)
 
     @classmethod
-    def render_from_file(cls, filename: str, context: Dict) -> str:
+    def render_from_file(cls, filename: str, context: dict[str, Any]) -> str:
         return FileRenderer.render_from_file(filename, context)
 
 
 class LaTeXRenderer(SphinxRenderer):
-    def __init__(self, template_path: str = None, latex_engine: str = None) -> None:
+    def __init__(self, template_path: Sequence[str | os.PathLike[str]] | None = None,
+                 latex_engine: str | None = None) -> None:
         if template_path is None:
-            template_path = os.path.join(package_dir, 'templates', 'latex')
+            template_path = [os.path.join(package_dir, 'templates', 'latex')]
         super().__init__(template_path)
 
         # use texescape as escape filter
@@ -84,10 +83,13 @@ class LaTeXRenderer(SphinxRenderer):
         self.env.variable_end_string = '%>'
         self.env.block_start_string = '<%'
         self.env.block_end_string = '%>'
+        self.env.comment_start_string = '<#'
+        self.env.comment_end_string = '#>'
 
 
 class ReSTRenderer(SphinxRenderer):
-    def __init__(self, template_path: Union[str, List[str]] = None, language: str = None) -> None:  # NOQA
+    def __init__(self, template_path: Sequence[str | os.PathLike[str]] | None = None,
+                 language: str | None = None) -> None:
         super().__init__(template_path)
 
         # add language to environment
@@ -102,8 +104,9 @@ class ReSTRenderer(SphinxRenderer):
 class SphinxTemplateLoader(BaseLoader):
     """A loader supporting template inheritance"""
 
-    def __init__(self, confdir: str, templates_paths: List[str],
-                 system_templates_paths: List[str]) -> None:
+    def __init__(self, confdir: str | os.PathLike[str],
+                 templates_paths: Sequence[str | os.PathLike[str]],
+                 system_templates_paths: Sequence[str | os.PathLike[str]]) -> None:
         self.loaders = []
         self.sysloaders = []
 
@@ -116,7 +119,7 @@ class SphinxTemplateLoader(BaseLoader):
             self.loaders.append(loader)
             self.sysloaders.append(loader)
 
-    def get_source(self, environment: Environment, template: str) -> Tuple[str, str, Callable]:
+    def get_source(self, environment: Environment, template: str) -> tuple[str, str, Callable]:
         if template.startswith('!'):
             # search a template from ``system_templates_paths``
             loaders = self.sysloaders
