@@ -8,19 +8,19 @@ from __future__ import annotations
 
 import re
 from collections import defaultdict
+from collections.abc import Iterable
 from os import path
-from typing import TYPE_CHECKING, Any, Iterable, cast
+from typing import TYPE_CHECKING, Any, cast
 
 from docutils import nodes, writers
-from docutils.nodes import Element, Node, Text
 
 from sphinx import addnodes, highlighting
-from sphinx.domains import IndexEntry
 from sphinx.domains.std import StandardDomain
 from sphinx.errors import SphinxError
 from sphinx.locale import _, __, admonitionlabels
-from sphinx.util import logging, split_into, texescape
+from sphinx.util import logging, texescape
 from sphinx.util.docutils import SphinxTranslator
+from sphinx.util.index_entries import split_index_msg
 from sphinx.util.nodes import clean_astext, get_prev_node
 from sphinx.util.template import LaTeXRenderer
 from sphinx.util.texescape import tex_replace_map
@@ -29,11 +29,14 @@ try:
     from docutils.utils.roman import toRoman
 except ImportError:
     # In Debian/Ubuntu, roman package is provided as roman, not as docutils.utils.roman
-    from roman import toRoman  # type: ignore
+    from roman import toRoman  # type: ignore[no-redef]
 
 if TYPE_CHECKING:
+    from docutils.nodes import Element, Node, Text
+
     from sphinx.builders.latex import LaTeXBuilder
     from sphinx.builders.latex.theming import Theme
+    from sphinx.domains import IndexEntry
 
 
 logger = logging.getLogger(__name__)
@@ -74,12 +77,11 @@ class LaTeXWriter(writers.Writer):
     ))
     settings_defaults: dict[str, Any] = {}
 
-    output = None
+    theme: Theme
 
     def __init__(self, builder: LaTeXBuilder) -> None:
         super().__init__()
         self.builder = builder
-        self.theme: Theme = None
 
     def translate(self) -> None:
         visitor = self.builder.create_translator(self.document, self.builder, self.theme)
@@ -109,17 +111,18 @@ class Table:
         elif 'colorrows' in self.classes:
             self.styles.append('colorrows')
         self.colcount = 0
-        self.colspec: str = None
-        self.colsep: str = None
+        self.colspec: str = ''
         if 'booktabs' in self.styles or 'borderless' in self.styles:
-            self.colsep = ''
+            self.colsep: str | None = ''
         elif 'standard' in self.styles:
             self.colsep = '|'
+        else:
+            self.colsep = None
         self.colwidths: list[int] = []
         self.has_problematic = False
         self.has_oldproblematic = False
         self.has_verbatim = False
-        self.caption: list[str] = None
+        self.caption: list[str] = []
         self.stubs: list[int] = []
 
         # current position
@@ -168,6 +171,7 @@ class Table:
             return self.colspec
 
         _colsep = self.colsep
+        assert _colsep is not None
         if self.colwidths and 'colwidths-given' in self.classes:
             total = sum(self.colwidths)
             colspecs = [r'\X{%d}{%d}' % (width, total) for width in self.colwidths]
@@ -659,6 +663,7 @@ class LaTeXTranslator(SphinxTranslator):
     def depart_title(self, node: Element) -> None:
         self.in_title = 0
         if isinstance(node.parent, nodes.table):
+            assert self.table is not None
             self.table.caption = self.popbody()
         else:
             self.body.append(self.context.pop())
@@ -700,11 +705,51 @@ class LaTeXTranslator(SphinxTranslator):
             self.body.append(CR + r'\end{fulllineitems}' + BLANKLINE)
 
     def _visit_signature_line(self, node: Element) -> None:
+        def next_sibling(e: Node) -> Node | None:
+            try:
+                return e.parent[e.parent.index(e) + 1]
+            except (AttributeError, IndexError):
+                return None
+
+        def has_multi_line(e: Element) -> bool:
+            return e.get('multi_line_parameter_list')
+
+        self.has_tp_list = False
+
         for child in node:
+            if isinstance(child, addnodes.desc_type_parameter_list):
+                self.has_tp_list = True
+                # recall that return annotations must follow an argument list,
+                # so signatures of the form "foo[tp_list] -> retann" will not
+                # be encountered (if they should, the `domains.python.py_sig_re`
+                # pattern must be modified accordingly)
+                arglist = next_sibling(child)
+                assert isinstance(arglist, addnodes.desc_parameterlist)
+                # tp_list + arglist: \macro{name}{tp_list}{arglist}{return}
+                multi_tp_list = has_multi_line(child)
+                multi_arglist = has_multi_line(arglist)
+
+                if multi_tp_list:
+                    if multi_arglist:
+                        self.body.append(CR + r'\pysigwithonelineperargwithonelinepertparg{')
+                    else:
+                        self.body.append(CR + r'\pysiglinewithargsretwithonelinepertparg{')
+                else:
+                    if multi_arglist:
+                        self.body.append(CR + r'\pysigwithonelineperargwithtypelist{')
+                    else:
+                        self.body.append(CR + r'\pysiglinewithargsretwithtypelist{')
+                break
+
             if isinstance(child, addnodes.desc_parameterlist):
-                self.body.append(CR + r'\pysiglinewithargsret{')
+                # arglist only: \macro{name}{arglist}{return}
+                if has_multi_line(child):
+                    self.body.append(CR + r'\pysigwithonelineperarg{')
+                else:
+                    self.body.append(CR + r'\pysiglinewithargsret{')
                 break
         else:
+            # no tp_list, no arglist: \macro{name}
             self.body.append(CR + r'\pysigline{')
 
     def _depart_signature_line(self, node: Element) -> None:
@@ -781,32 +826,117 @@ class LaTeXTranslator(SphinxTranslator):
     def depart_desc_returns(self, node: Element) -> None:
         self.body.append(r'}')
 
+    def _visit_sig_parameter_list(self, node: Element, parameter_group: type[Element]) -> None:
+        """Visit a signature parameters or type parameters list.
+
+        The *parameter_group* value is the type of a child node acting as a required parameter
+        or as a set of contiguous optional parameters.
+
+        The caller is responsible for closing adding surrounding LaTeX macro argument start
+        and stop tokens.
+        """
+        self.is_first_param = True
+        self.optional_param_level = 0
+        self.params_left_at_level = 0
+        self.param_group_index = 0
+        # Counts as what we call a parameter group either a required parameter, or a
+        # set of contiguous optional ones.
+        self.list_is_required_param = [isinstance(c, parameter_group) for c in node.children]
+        # How many required parameters are left.
+        self.required_params_left = sum(self.list_is_required_param)
+        self.param_separator = r'\sphinxparamcomma '
+        self.multi_line_parameter_list = node.get('multi_line_parameter_list', False)
+
     def visit_desc_parameterlist(self, node: Element) -> None:
-        # close name, open parameterlist
-        self.body.append('}{')
-        self.first_param = 1
+        if not self.has_tp_list:
+            # close name argument (#1), open parameters list argument (#2)
+            self.body.append('}{')
+        self._visit_sig_parameter_list(node, addnodes.desc_parameter)
 
     def depart_desc_parameterlist(self, node: Element) -> None:
         # close parameterlist, open return annotation
         self.body.append('}{')
 
-    def visit_desc_parameter(self, node: Element) -> None:
-        if not self.first_param:
-            self.body.append(', ')
-        else:
-            self.first_param = 0
-        if not node.hasattr('noemph'):
-            self.body.append(r'\sphinxparam{')
+    def visit_desc_type_parameter_list(self, node: Element) -> None:
+        # close name argument (#1), open type parameters list argument (#2)
+        self.body.append('}{')
+        self._visit_sig_parameter_list(node, addnodes.desc_type_parameter)
 
-    def depart_desc_parameter(self, node: Element) -> None:
+    def depart_desc_type_parameter_list(self, node: Element) -> None:
+        # close type parameters list, open parameters list argument (#3)
+        self.body.append('}{')
+
+    def _visit_sig_parameter(self, node: Element, parameter_macro: str) -> None:
+        if self.is_first_param:
+            self.is_first_param = False
+        elif not self.multi_line_parameter_list and not self.required_params_left:
+            self.body.append(self.param_separator)
+        if self.optional_param_level == 0:
+            self.required_params_left -= 1
+        else:
+            self.params_left_at_level -= 1
+        if not node.hasattr('noemph'):
+            self.body.append(parameter_macro)
+
+    def _depart_sig_parameter(self, node: Element) -> None:
         if not node.hasattr('noemph'):
             self.body.append('}')
+        is_required = self.list_is_required_param[self.param_group_index]
+        if self.multi_line_parameter_list:
+            is_last_group = self.param_group_index + 1 == len(self.list_is_required_param)
+            next_is_required = (
+                not is_last_group
+                and self.list_is_required_param[self.param_group_index + 1]
+            )
+            opt_param_left_at_level = self.params_left_at_level > 0
+            if opt_param_left_at_level or is_required and (is_last_group or next_is_required):
+                self.body.append(self.param_separator)
+
+        elif self.required_params_left:
+            self.body.append(self.param_separator)
+
+        if is_required:
+            self.param_group_index += 1
+
+    def visit_desc_parameter(self, node: Element) -> None:
+        self._visit_sig_parameter(node, r'\sphinxparam{')
+
+    def depart_desc_parameter(self, node: Element) -> None:
+        self._depart_sig_parameter(node)
+
+    def visit_desc_type_parameter(self, node: Element) -> None:
+        self._visit_sig_parameter(node, r'\sphinxtypeparam{')
+
+    def depart_desc_type_parameter(self, node: Element) -> None:
+        self._depart_sig_parameter(node)
 
     def visit_desc_optional(self, node: Element) -> None:
-        self.body.append(r'\sphinxoptional{')
+        self.params_left_at_level = sum([isinstance(c, addnodes.desc_parameter)
+                                         for c in node.children])
+        self.optional_param_level += 1
+        self.max_optional_param_level = self.optional_param_level
+        if self.multi_line_parameter_list:
+            if self.is_first_param:
+                self.body.append(r'\sphinxoptional{')
+            elif self.required_params_left:
+                self.body.append(self.param_separator)
+                self.body.append(r'\sphinxoptional{')
+            else:
+                self.body.append(r'\sphinxoptional{')
+                self.body.append(self.param_separator)
+        else:
+            self.body.append(r'\sphinxoptional{')
 
     def depart_desc_optional(self, node: Element) -> None:
+        self.optional_param_level -= 1
+        if self.multi_line_parameter_list:
+            # If it's the first time we go down one level, add the separator before the
+            # bracket.
+            if self.optional_param_level == self.max_optional_param_level - 1:
+                self.body.append(self.param_separator)
         self.body.append('}')
+        if self.optional_param_level == 0:
+            self.param_group_index += 1
 
     def visit_desc_annotation(self, node: Element) -> None:
         self.body.append(r'\sphinxbfcode{\sphinxupquote{')
@@ -866,6 +996,7 @@ class LaTeXTranslator(SphinxTranslator):
 
     def visit_table(self, node: Element) -> None:
         if len(self.tables) == 1:
+            assert self.table is not None
             if self.table.get_table_type() == 'longtable':
                 raise UnsupportedError(
                     '%s:%s: longtable does not support nesting a table.' %
@@ -878,26 +1009,28 @@ class LaTeXTranslator(SphinxTranslator):
                 '%s:%s: deeply nested tables are not implemented.' %
                 (self.curfilestack[-1], node.line or ''))
 
-        self.tables.append(Table(node))
-        if self.table.colsep is None:
-            self.table.colsep = '' if (
-                'booktabs' in self.builder.config.latex_table_style or
-                'borderless' in self.builder.config.latex_table_style
-            ) else '|'
+        table = Table(node)
+        self.tables.append(table)
+        if table.colsep is None:
+            table.colsep = '|' * (
+                'booktabs' not in self.builder.config.latex_table_style
+                and 'borderless' not in self.builder.config.latex_table_style
+            )
         if self.next_table_colspec:
-            self.table.colspec = '{%s}' % self.next_table_colspec + CR
-            if '|' in self.table.colspec:
-                self.table.styles.append('vlines')
-                self.table.colsep = '|'
+            table.colspec = '{%s}' % self.next_table_colspec + CR
+            if '|' in table.colspec:
+                table.styles.append('vlines')
+                table.colsep = '|'
             else:
-                self.table.styles.append('novlines')
-                self.table.colsep = ''
+                table.styles.append('novlines')
+                table.colsep = ''
             if 'colwidths-given' in node.get('classes', []):
                 logger.info(__('both tabularcolumns and :widths: option are given. '
                                ':widths: is ignored.'), location=node)
         self.next_table_colspec = None
 
     def depart_table(self, node: Element) -> None:
+        assert self.table is not None
         labels = self.hypertarget_to(node)
         table_type = self.table.get_table_type()
         table = self.render(table_type + '.tex_t',
@@ -909,6 +1042,7 @@ class LaTeXTranslator(SphinxTranslator):
         self.tables.pop()
 
     def visit_colspec(self, node: Element) -> None:
+        assert self.table is not None
         self.table.colcount += 1
         if 'colwidth' in node:
             self.table.colwidths.append(node['colwidth'])
@@ -925,6 +1059,7 @@ class LaTeXTranslator(SphinxTranslator):
         pass
 
     def visit_thead(self, node: Element) -> None:
+        assert self.table is not None
         # Redirect head output until header is finished.
         self.pushbody(self.table.header)
 
@@ -934,6 +1069,7 @@ class LaTeXTranslator(SphinxTranslator):
         self.popbody()
 
     def visit_tbody(self, node: Element) -> None:
+        assert self.table is not None
         # Redirect body output until table is finished.
         self.pushbody(self.table.body)
 
@@ -943,6 +1079,7 @@ class LaTeXTranslator(SphinxTranslator):
         self.popbody()
 
     def visit_row(self, node: Element) -> None:
+        assert self.table is not None
         self.table.col = 0
         _colsep = self.table.colsep
         # fill columns if the row starts with the bottom of multirow cell
@@ -962,9 +1099,11 @@ class LaTeXTranslator(SphinxTranslator):
                                  (cell.width, _colsep, _colsep, cell.cell_id))
 
     def depart_row(self, node: Element) -> None:
+        assert self.table is not None
         self.body.append(r'\\' + CR)
         cells = [self.table.cell(self.table.row, i) for i in range(self.table.colcount)]
-        underlined = [cell.row + cell.height == self.table.row + 1 for cell in cells]
+        underlined = [cell.row + cell.height == self.table.row + 1  # type: ignore[union-attr]
+                      for cell in cells]
         if all(underlined):
             self.body.append(r'\sphinxhline')
         else:
@@ -973,7 +1112,7 @@ class LaTeXTranslator(SphinxTranslator):
             if underlined[0] is False:
                 i = 1
                 while i < self.table.colcount and underlined[i] is False:
-                    if cells[i - 1].cell_id != cells[i].cell_id:
+                    if cells[i - 1].cell_id != cells[i].cell_id:  # type: ignore[union-attr]
                         self.body.append(r'\sphinxvlinecrossing{%d}' % i)
                     i += 1
             while i < self.table.colcount:
@@ -983,17 +1122,19 @@ class LaTeXTranslator(SphinxTranslator):
                 i += j
                 i += 1
                 while i < self.table.colcount and underlined[i] is False:
-                    if cells[i - 1].cell_id != cells[i].cell_id:
+                    if cells[i - 1].cell_id != cells[i].cell_id:  # type: ignore[union-attr]
                         self.body.append(r'\sphinxvlinecrossing{%d}' % i)
                     i += 1
             self.body.append(r'\sphinxfixclines{%d}' % self.table.colcount)
         self.table.row += 1
 
     def visit_entry(self, node: Element) -> None:
+        assert self.table is not None
         if self.table.col > 0:
             self.body.append('&')
         self.table.add_cell(node.get('morerows', 0) + 1, node.get('morecols', 0) + 1)
         cell = self.table.cell()
+        assert cell is not None
         context = ''
         _colsep = self.table.colsep
         if cell.width > 1:
@@ -1040,7 +1181,9 @@ class LaTeXTranslator(SphinxTranslator):
 
         self.body.append(self.context.pop())
 
+        assert self.table is not None
         cell = self.table.cell()
+        assert cell is not None
         self.table.col += cell.width
         _colsep = self.table.colsep
 
@@ -1357,8 +1500,9 @@ class LaTeXTranslator(SphinxTranslator):
                 length = self.latex_image_length(node['width'])
             elif isinstance(node[0], nodes.image) and 'width' in node[0]:
                 length = self.latex_image_length(node[0]['width'])
-            self.body.append(BLANKLINE)     # Insert a blank line to prevent infinite loop
-                                            # https://github.com/sphinx-doc/sphinx/issues/7059
+            # Insert a blank line to prevent an infinite loop
+            # https://github.com/sphinx-doc/sphinx/issues/7059
+            self.body.append(BLANKLINE)
             self.body.append(r'\begin{wrapfigure}{%s}{%s}' %
                              ('r' if node['align'] == 'right' else 'l', length or '0pt') + CR)
             self.body.append(r'\centering')
@@ -1501,8 +1645,8 @@ class LaTeXTranslator(SphinxTranslator):
             if has_dup_label(prev):
                 ids = node['ids'][:]  # copy to avoid side-effects
                 while has_dup_label(prev):
-                    ids.remove(prev['refid'])  # type: ignore
-                    prev = get_prev_node(prev)
+                    ids.remove(prev['refid'])  # type: ignore[index]
+                    prev = get_prev_node(prev)  # type: ignore[arg-type]
             else:
                 ids = iter(node['ids'])  # read-only iterator
         else:
@@ -1547,37 +1691,32 @@ class LaTeXTranslator(SphinxTranslator):
             if ismain:
                 m = '|spxpagem'
             try:
+                parts = tuple(map(escape, split_index_msg(type, string)))
+                styled = tuple(map(style, parts))
                 if type == 'single':
                     try:
-                        p1, p2 = (escape(x) for x in split_into(2, 'single', string))
-                        P1, P2 = style(p1), style(p2)
+                        p1, p2 = parts
+                        P1, P2 = styled
                         self.body.append(fr'\index{{{p1}@{P1}!{p2}@{P2}{m}}}')
                     except ValueError:
-                        p = escape(split_into(1, 'single', string)[0])
-                        P = style(p)
+                        p, = parts
+                        P, = styled
                         self.body.append(fr'\index{{{p}@{P}{m}}}')
                 elif type == 'pair':
-                    p1, p2 = (escape(x) for x in split_into(2, 'pair', string))
-                    P1, P2 = style(p1), style(p2)
-                    self.body.append(r'\index{%s@%s!%s@%s%s}\index{%s@%s!%s@%s%s}' %
-                                     (p1, P1, p2, P2, m, p2, P2, p1, P1, m))
+                    p1, p2 = parts
+                    P1, P2 = styled
+                    self.body.append(fr'\index{{{p1}@{P1}!{p2}@{P2}{m}}}'
+                                     fr'\index{{{p2}@{P2}!{p1}@{P1}{m}}}')
                 elif type == 'triple':
-                    p1, p2, p3 = (escape(x) for x in split_into(3, 'triple', string))
-                    P1, P2, P3 = style(p1), style(p2), style(p3)
+                    p1, p2, p3 = parts
+                    P1, P2, P3 = styled
                     self.body.append(
-                        r'\index{%s@%s!%s %s@%s %s%s}'
-                        r'\index{%s@%s!%s, %s@%s, %s%s}'
-                        r'\index{%s@%s!%s %s@%s %s%s}' %
-                        (p1, P1, p2, p3, P2, P3, m,
-                         p2, P2, p3, p1, P3, P1, m,
-                         p3, P3, p1, p2, P1, P2, m))
-                elif type == 'see':
-                    p1, p2 = (escape(x) for x in split_into(2, 'see', string))
-                    P1 = style(p1)
-                    self.body.append(fr'\index{{{p1}@{P1}|see{{{p2}}}}}')
-                elif type == 'seealso':
-                    p1, p2 = (escape(x) for x in split_into(2, 'seealso', string))
-                    P1 = style(p1)
+                        fr'\index{{{p1}@{P1}!{p2} {p3}@{P2} {P3}{m}}}'
+                        fr'\index{{{p2}@{P2}!{p3}, {p1}@{P3}, {P1}{m}}}'
+                        fr'\index{{{p3}@{P3}!{p1} {p2}@{P1} {P2}{m}}}')
+                elif type in {'see', 'seealso'}:
+                    p1, p2 = parts
+                    P1, _P2 = styled
                     self.body.append(fr'\index{{{p1}@{P1}|see{{{p2}}}}}')
                 else:
                     logger.warning(__('unknown index entry type %s found'), type)
